/*****************************************************************************
*              	     VS1053b record test progarm
* 
******************************************************************************/
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "MMC/mmc.h"
#include "VS1053B/vs1053b.h"
#include "MYUTIL/timer.h"
#include "FatFs/ff.h"

#define LED_ON()  { PORTD |=  _BV(PD6); }
#define LED_OFF() { PORTD &= ~_BV(PD6); }

#define SW1 ( PIND & _BV(PD2) )
#define SW2 ( PIND & _BV(PD3) )

/*****************************************************************************
*				timer0(8bit) Interruption at intervals ( 10msec )
*****************************************************************************/
ISR( TIMER0_COMPA_vect )
{
	disk_timerproc();								/* FatFs timer			*/
}

/*****************************************************************************
*							hardware initialization
* In : void
* Out: void
*****************************************************************************/
static void hardwareInit( void )
{
	/*  PortB +--------7: HI-Z   : XTAL                                     */
	/*        |+-------6: HI-Z   : XTAL                                     */
	/*        ||+------5: Out(H) : VS1053b SCK                              */
	/*        |||+-----4: HIZ    : VS1053b SO			                    */
	/*        ||||+----3: Out(L) : VS1053b SI			                    */
	/*        |||||+---2: Out(H) : VS1053b #XCS                             */
	/*        ||||||+--1: Out(H) : VS1053b #XDCS                            */
	/*        |||||||+-0: Out(L) : VS1053b #RESET                           */
	DDRB  = 0b11101111;
	PORTB = 0b00100110;

	/*  PortC +--------7: Pullup : non use	                                */
	/*        |+-------6: Pullup : non use                                  */
	/*        ||+------5: Pullup : non use				      	 	        */
	/*        |||+-----4: Pullup : non use                                  */
	/*        ||||+----3: Pullup : non use                                  */
	/*        |||||+---2: Pullup : SD #INS                                  */
	/*        ||||||+--1: OUT(H) : SD #POW                                  */
	/*        |||||||+-0: OUT(H) : SD #CS                                   */
	DDRC  = 0b00000011;
	PORTC = 0b11110111;

	/* 	PortD +--------7: HI-Z   : VS1053b DREQ                             */
	/*        |+-------6: Out(L) : LED                                      */
	/*        ||+------5: Pullup : non use                                  */
	/*        |||+-----4: Out(L) : SD-CARD CLK                              */
	/*        ||||+----3: Pullup : SW2                                      */
	/*        |||||+---2: Pullup : SW1		                                */
	/*        ||||||+--1: Out(L) : SD-CARD SI   	                        */
	/*        |||||||+-0: HIZ    : SD-CARD SO 	                            */
	DDRD  = 0b01010010;
	PORTD = 0b00101100;

	/* set timer 0		*/
	OCR0A =  TIMER_MS_TO_COUNT( 10 );		/* 10msec						*/
	TIMSK0 |= _BV(OCIE0A);					/* enable interrupt				*/
	TCCR0A = _BV(WGM01);					/* CTC mode						*/
	TCCR0B =  0x05;							/* F_CPU / 1024					*/

	/* VS1053b SPI initialization	*/
	VS1053_initialization();

	sei();									/* enable global interrupt		*/
}

/*****************************************************************************
*								AD-PCM RECORD
*****************************************************************************/
uint8_t vs1053_Recording_ADPCM( FIL *file )
{
	const uint32_t freq = 16000;			/* sample freq		Unalterable	*/
	const uint16_t ch = 2;					/* 1:mono 2:Stereo	Unalterable	*/

	uint16_t block_byte_size = ch * 256;
	uint16_t block_word_size = ch * 128;

	uint16_t n;
	uint32_t blockIndex ,data;
	uint8_t wavebuffer[512];

	/* WAVE file header struct	*/
	struct header_t {
		char ChunkID[4];					/* "RIFF"						*/
		uint32_t ChunkSize;					/* FileSize - 8					*/
		char Format[4];						/* "WAVE"						*/
		char SubChunk1ID[4];				/* "fmt "						*/
		uint32_t SubChunk1Size;				/* AD-PCM = 20					*/
		uint16_t AudioFormat;				/* AD-PCM = 0x11				*/
		uint16_t NumOfChannels;				/* MONO = 1 / ST = 2			*/
		uint32_t SampleRate;				/* Sample Freq					*/
		uint32_t ByteRate;					/* BFs~Ch~256 / 505			*/
		uint16_t BlockAlign;				/* MONO = 256 / ST = 512		*/
		uint16_t BitsPerSample;				/* 4bitADPCM = 0x04				*/
		uint16_t ByteExtraData;				/* 2							*/
		uint16_t ExtraData;					/* BlockSize = 505				*/
		char SubChunk2ID[4];				/* "fact"						*/
		uint32_t SubChunk2Size;				/* 4							*/
		uint32_t NumOfSamples;				/* n * 505						*/
		char SubChunk3ID[4];				/* "data"   					*/
		uint32_t SubChunk3Size;				/* FileSize - 60				*/
	} header;

	/* start recording			*/
	if ( f_lseek( file ,sizeof(header) ) != FR_OK ) {
		return ( 1 );
	}

	/* VS1053b hard reset				*/
	VS_ENABLE_XRESET();											/* reset pulse		*/
	WAIT_MS( 4 );												/* 4msec wait		*/
	VS_DISABLE_XRESET();
	WAIT_MS( 4 );												/* 4msec wait		*/

	/* VS1053b set clock 				*/
	VS1053_write( SCI_CLOCKF ,VS_MULTIPLE35 );
	WAIT_MS( 4 );												/* 4msec wait		*/

	/* VS1053b soft reset				*/
	VS1053_write( SCI_MODE ,SM_SDINEW | SM_RESET );
	WAIT_MS( 4 );												/* 4msec wait		*/

	/* load vs1053b-patches.plg			*/
	VS1053_patches150();

	VS1053_write( SCI_BASS , 0x0000 );
	VS1053_write( SCI_AICTRL0 ,freq - 4096 );	/* It is 4096 why is not understood well. ??? */
	VS1053_write( SCI_AICTRL1 ,0);
	VS1053_write( SCI_AICTRL2 ,4096U );
	VS1053_write( SCI_AICTRL3 ,0000U );
	VS1053_write( SCI_MODE ,SM_SDINEW | SM_ADPCM | SM_LINE1 );
	VS1053_write( SCI_AIADDR ,0x0300 );

	/* main loop				*/
	blockIndex = 0;
	while ( SW2 ) {							/* SW2 non push ?						*/
		n = VS1053_read( SCI_HDAT1 );		/* get wave data length					*/
		if ( n < block_word_size ) continue;
 
		for ( uint16_t i = 0 ;i < block_byte_size ;i += 2 ) {
			data = VS1053_read( SCI_HDAT0 );
			wavebuffer[ i + 0 ] = (uint8_t)( data >> 8 );
			wavebuffer[ i + 1 ] = (uint8_t)data;
		}

		if ( ( f_write( file ,wavebuffer ,block_byte_size ,&n ) != FR_OK ) || ( n != block_byte_size ) ) {
			return ( 1 );
		}

		blockIndex++;
	}

	/* write WAVE file header	*/
	memcpy_P( header.ChunkID ,PSTR("RIFF") ,4 );
	header.ChunkSize = blockIndex * (uint32_t)ch * 256L + 52L;
	memcpy_P( header.Format ,PSTR("WAVE") ,4 );
	memcpy_P( header.SubChunk1ID ,PSTR("fmt ") ,4 );
	header.SubChunk1Size = 0x14;
	header.AudioFormat = 0x11;
	header.NumOfChannels = ch;
	header.SampleRate = freq;
	header.ByteRate = freq * (uint32_t)ch * 256L / 505L;
	header.BlockAlign = block_byte_size;
	header.BitsPerSample = 4;
	header.ByteExtraData = 2;
	header.ExtraData = 505;
	memcpy_P( header.SubChunk2ID ,PSTR("fact") ,4 );
	header.SubChunk2Size = 4;
	header.NumOfSamples = blockIndex * 505L;
	memcpy_P( header.SubChunk3ID ,PSTR("data") ,4 );
	header.SubChunk3Size = blockIndex * (uint32_t)ch * 256L;

	if ( ( f_lseek( file ,0L ) != FR_OK ) ) {
		return ( 1 );
	}

	if ( ( f_write( file ,&header ,sizeof(header) ,&n ) != FR_OK ) || ( n != sizeof(header) ) ) {
		return ( 1 );
	}

	return ( 0 );
}

/*****************************************************************************
*									 main
*****************************************************************************/
int main( void )
{
	char fname[16];
	FATFS fs;
	FIL file;

	hardwareInit();
	strcpy_P( fname ,PSTR("test.wav") );

	while ( 1 ) {
		if ( !SW1 ) {
			LED_ON();
			f_mount( 0 ,&fs );
			if ( f_open( &file ,fname ,FA_WRITE | FA_CREATE_ALWAYS ) == FR_OK ) {
				vs1053_Recording_ADPCM( &file );
			}
			VS1053_write( SCI_MODE ,SM_SDINEW | SM_RESET );
			f_close( &file );
			LED_OFF();
			while ( !SW1 );
		}
	}

	return ( 0 );
}

